﻿#include <windows.h>
#include <stdio.h>

#define ID_BUTTON1 1001
#define ID_BUTTON2 1002
#define ID_TEXTBOX 1003

#define BUFFER_SIZE 10

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);
DWORD WINAPI ProducerThread(LPVOID lpParam);
DWORD WINAPI ConsumerThread(LPVOID lpParam);

int buffer[BUFFER_SIZE];
int in = 0;
int out = 0;
int count = 0;

HANDLE hMutex;
HANDLE hFullSemaphore;
HANDLE hEmptySemaphore;

void AppendTextToTextBox(HWND hwndTextBox, const char* text) {
    int length = GetWindowTextLength(hwndTextBox);
    SendMessage(hwndTextBox, EM_SETSEL, (WPARAM)length, (LPARAM)length);
    SendMessage(hwndTextBox, EM_REPLACESEL, 0, (LPARAM)text);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow) {
    const char* className = "MyWindowClass";
    WNDCLASSEX wc;
    HWND hwnd;
    MSG Msg;

    wc.cbSize        = sizeof(WNDCLASSEX);
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = WndProc;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = className;
    wc.hIconSm       = LoadIcon(NULL, IDI_APPLICATION);

    if(!RegisterClassEx(&wc)) {
        MessageBox(NULL, "窗口注册失败！", "错误！",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    hwnd = CreateWindowEx(
        WS_EX_CLIENTEDGE,
        className,
        "生产者-消费者 示例",
        WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, 450, 300,
        NULL, NULL, hInstance, NULL);

    if(hwnd == NULL) {
        MessageBox(NULL, "窗口创建失败！", "错误！",
            MB_ICONEXCLAMATION | MB_OK);
        return 0;
    }

    // 初始化信号量和互斥体
    hMutex = CreateMutex(NULL, FALSE, NULL);
    hFullSemaphore = CreateSemaphore(NULL, 0, BUFFER_SIZE, NULL);
    hEmptySemaphore = CreateSemaphore(NULL, BUFFER_SIZE, BUFFER_SIZE, NULL);

    ShowWindow(hwnd, nCmdShow);
    UpdateWindow(hwnd);

    while(GetMessage(&Msg, NULL, 0, 0) > 0) {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
    }
    CloseHandle(hMutex);
    CloseHandle(hFullSemaphore);
    CloseHandle(hEmptySemaphore);
    return Msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam) {
    static HWND button1, button2, textBox;
    switch(msg) {
        case WM_CREATE:
            button1 = CreateWindow("BUTTON", "启动生产者",
                WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON,
                10, 10, 200, 30, hwnd, (HMENU)ID_BUTTON1, ((LPCREATESTRUCT)lParam)->hInstance, NULL);
            button2 = CreateWindow("BUTTON", "启动消费者",
                WS_TABSTOP | WS_VISIBLE | WS_CHILD | BS_DEFPUSHBUTTON,
                10, 50, 200, 30, hwnd, (HMENU)ID_BUTTON2, ((LPCREATESTRUCT)lParam)->hInstance, NULL);
            textBox = CreateWindow("EDIT", "",
                WS_BORDER | WS_CHILD | WS_VISIBLE | ES_MULTILINE | ES_READONLY,
                10, 90, 420, 180, hwnd, (HMENU)ID_TEXTBOX, ((LPCREATESTRUCT)lParam)->hInstance, NULL);
            break;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case ID_BUTTON1:
                    CreateThread(NULL, 0, ProducerThread, (LPVOID)textBox, 0, NULL);
                    break;
                case ID_BUTTON2:
                    CreateThread(NULL, 0, ConsumerThread, (LPVOID)textBox, 0, NULL);
                    break;
            }
            break;
        case WM_DESTROY:
            PostQuitMessage(0);
            break;
        default:
            return DefWindowProc(hwnd, msg, wParam, lParam);
    }
    return 0;
}

DWORD WINAPI ProducerThread(LPVOID lpParam) {
    HWND hwndTextBox = (HWND)lpParam;
    char buffer[256];

    for(int i = 0; i < 20; i++) {
        WaitForSingleObject(hEmptySemaphore, INFINITE);
        WaitForSingleObject(hMutex, INFINITE);

        // 生产产品
        int item = rand() % 100;
        buffer[in] = item;
        in = (in + 1) % BUFFER_SIZE;
        count++;

        sprintf(buffer, "生产者生产了: %d\n", item);
        AppendTextToTextBox(hwndTextBox, buffer);

        ReleaseMutex(hMutex);
        ReleaseSemaphore(hFullSemaphore, 1, NULL);

        Sleep(rand() % 1000); // 模拟生产时间
    }
    return 0;
}

DWORD WINAPI ConsumerThread(LPVOID lpParam) {
    HWND hwndTextBox = (HWND)lpParam;
    char buffer[256];

    for(int i = 0; i < 20; i++) {
        WaitForSingleObject(hFullSemaphore, INFINITE);
        WaitForSingleObject(hMutex, INFINITE);

        // 消费产品
        int item = buffer[out];
        out = (out + 1) % BUFFER_SIZE;
        count--;

        sprintf(buffer, "消费者消费了: %d\n", item);
        AppendTextToTextBox(hwndTextBox, buffer);

        ReleaseMutex(hMutex);
        ReleaseSemaphore(hEmptySemaphore, 1, NULL);

        Sleep(rand() % 1000); // 模拟消费时间
    }
    return 0;
}


